//********************************************************************
//*** ResourcePool/mysql/Factory.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: Factory.cc,v 1.9 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_MYSQL_FACTORY_HH
#include "Factory.hh"
#endif

#ifndef RESOURCEPOOL_CREATEXCEPTION_HH
#include "ResourcePool/CreateException.hh"
#endif

#include "lex.mysql_connect.hh"

#include <string>

namespace fatalmind {

mysqlFactory::mysqlFactory(const std::string& host
            , const std::string& user
            , const std::string& pass
            , const std::string& db
            , unsigned int port
            , const std::string& sock
)
: _host(host) 
, _user(user)
, _pass(pass)
, _db(db)
, _port(port)
, _sock(sock)
{
}

mysqlFactory::mysqlFactory(const std::string& connect)
{
	mysql_connect mc(connect);
	_host = mc.getHost();
	_user = mc.getUser();
	_pass = mc.getPass();
	_db   = mc.getDb();
	_port = mc.getPort();
}

Clone* mysqlFactory::DoClone() const {
	return new mysqlFactory(*this);
}

mysqlResource* mysqlFactory::create_resource() const
{
	return new mysqlResource(_host, _user, _pass, _db, _port, _sock);
}

} // namespace fatalmind
