//********************************************************************
//*** ResourcePool/mysql/Resource.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.hh,v 1.7 2007/10/09 19:55:09 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_MYSQL_RESOURCE_HH
#define RESOURCEPOOL_MYSQL_RESOURCE_HH

#ifndef RESOURCEPOOL_MYSQL_MYSQLFWD_HH
#include "mysqlFwd.hh"
#endif

#include <string>

namespace fatalmind {
namespace mysql {
class FinallyTxDepth;
}; // namespace mysql

class mysqlResource {
	public:
		mysqlResource(const std::string& host
			, const std::string& user
			, const std::string& pass
			, const std::string& db
			, unsigned int port = 3306
			, const std::string& sock = ""
		);
		~mysqlResource();
		bool precheck();
		bool postcheck();
		void close();
		void fail_close();
		MYSQL& get_plain_resource();

		int getTxDepth() {
			return _txDepth;
		}

	private:
		MYSQL*   _dbh;
	// transaction managment helper function
		int _txDepth;
		int incTxDepth() {
			return _txDepth++;
		}
		void decTxDepth() {
			--_txDepth;
		}

	friend class mysql::FinallyTxDepth;
};

namespace mysql {
// TODO, use a more generic approach
class FinallyTxDepth {
	public:
		FinallyTxDepth(mysqlResource& a_hand)
		: hand(a_hand)
		{
			hand.incTxDepth();
		}

		~FinallyTxDepth() {
			hand.decTxDepth();
		}
	private:
		mysqlResource& hand;
};
}; // namespace mysql

} // namespace fatalmind
#endif
