//********************************************************************
//*** ResourcePool/mysql/SQLExecute.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLExecute.cc,v 1.20 2007/09/19 12:42:38 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#include "SQLExecute.hh"

#include "lex.mysql_bind.hh"
//#include "ResourcePool/ResourcePool.hh"

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_HH
#include "SQLType.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_MYSQLEXCEPTION_HH
#include "MysqlException.hh"
#endif

#include "ResourcePool/SQL/Bind.cc"
//#include "SQLType.cc"

#include <mysql/mysql.h>

namespace fatalmind {
namespace mysql {

SQLExecute::SQLExecute(const std::string& SQL)
: _SQL(SQL)
{
}

SQLExecute::SQLExecute(const ResourcePool<ResourceType>& p, const std::string& SQL)
: _SQL(SQL)
{
}

SQLExecute::~SQLExecute()
{
}

Clone* SQLExecute::DoClone() const 
{
	return new SQLExecute(*this);
}

void SQLExecute::execute(mysqlResource& rdbh)
{
	MYSQL& dbh = rdbh.get_plain_resource();
	mysql_bind boundsql(dbh, _SQL, _args);
	if (int rc = mysql_real_query(&dbh
		, boundsql.getSQL().c_str()
		, boundsql.getSQL().length())
	) {
		MysqlException::throwit(rc, mysql_error(&dbh), mysql_errno(&dbh), _SQL);
	}
}

} // namespace mysql

template class
SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>;

template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<char>
	(const unsigned int pos, const char& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<signed char>
	(const unsigned int pos, const signed char& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<unsigned char>
	(const unsigned int pos, const unsigned char& val, bool nul);

template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<short int>
	(const unsigned int pos, const short int& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<int>	
	(const unsigned int pos, const int& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<long int>
	(const unsigned int pos, const long int& val, bool nul);

template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<unsigned short int>
	(const unsigned int pos, unsigned const short int& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<unsigned int>
	(const unsigned int pos, unsigned const int& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<unsigned long int>
	(const unsigned int pos, unsigned const long int& val, bool nul);

// wchar_t, bool, not supported..

template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<float>
	(const unsigned int pos, const float& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<double>
	(const unsigned int pos, const double& val, bool nul);
template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<long double>
	(const unsigned int pos, const long double& val, bool nul);

template void SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>::bind<std::string>
	(const unsigned int pos, const std::string& val, bool nul);



} // namespace fatalmind
