//********************************************************************
//*** ResourcePool/oracle/ResourcePool.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePool.hh,v 1.24 2007/09/19 12:42:38 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_RESOURCEPOOL_HH
#define RESOURCEPOOL_ORACLE_RESOURCEPOOL_HH

#ifndef RESOURCEPOOL_ORACLE_FACTORY_HH
#include "ResourcePool/oracle/Factory.hh"
#endif

#ifndef RESOURCEPOOL_RESOURCEPOOL_HH
#include "ResourcePool/ResourcePool.hh"
#endif

#include "ResourcePool/BatchCommand.hh"

#ifndef RESOURCEPOOL_ORACLE_SQLEXECUTE_HH
#include "SQLExecute.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SQLSELECT_HH
#include "SQLSelect.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SQLSELECTROW_HH
#include "SQLSelectRow.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_TRANSACTION_HH
#include "Transaction.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTION_HH
#include "OracleException.hh"
#endif

#ifndef RESOURCEPOOL_SQL_SQLException_HH
#include "ResourcePool/SQL/SQLException.hh"
#endif

#ifndef INCLUDED_THREAD_LOCK_HH
#include "Thread/Lock.hh"
#endif

#include "ResourceFwd.hh"

namespace fatalmind {

/**
 * the fatalmind::oracle namespace contains
 * all classes required to implement the oracle
 * ResourceType.
 */
namespace oracle {

template<class TM, template<class> class SLMDC>
struct Commands;

/**
 * The pool type to access oracle databases.
 */ 
template<class TM, template<class>class SLMDC>
class ResourceType
: public PoolType<oracleFactory<TM, SLMDC>
	, oracleResource<TM>
	, oracleHandlesWithCache<TM>
	, Commands<TM, SLMDC>
	, TM>
{
};

template<class TM, template<class> class SLMDC>
struct Commands {
	Commands(const oracleFactory<TM, SLMDC>& f) {
	};

	typedef fatalmind::oracle::SQLExecute<TM, SLMDC>   SQLExecute;
	typedef fatalmind::oracle::SQLSelect<TM, SLMDC>    SQLSelect;
	typedef fatalmind::oracle::SQLSelectRow<TM, SLMDC> SQLSelectRow;
	typedef fatalmind::BatchCommand<Command<ResourceType<TM> > > SQLBatch;
	typedef fatalmind::oracle::Transaction<TM, SLMDC>  Transaction;
};

} // namespace oracle

namespace ResourceType {

typedef oracle::ResourceType<DefaultThreadedModel, oracle::SelectListMetaDataNopCache> oracleNopCache;
typedef oracle::ResourceType<DefaultThreadedModel, oracle::SelectListMetaDataCache> oracle;

/*
typedef oracle::ResourceType<SingleThreadedModel<>, oracle::SelectListMetaDataNopCache> oracleNopCacheST;
typedef oracle::ResourceType<SingleThreadedModel<>, oracle::SelectListMetaDataCache> oracleST;

typedef oracle::ResourceType<MultiThreadedModel<>, oracle::SelectListMetaDataNopCache> oracleNopCacheMT;
typedef oracle::ResourceType<MultiThreadedModel<>, oracle::SelectListMetaDataCache> oracleMT;
*/

} // namespace ResourceType
} // namespace fatalmind

#include "Resource.hh"

#endif

