//********************************************************************
//*** Thread/CondVar.t.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: CondVar.t.cc,v 1.13 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "Thread/Thread.hh"
#include "Thread/CondVar.hh"
#include "Exceptions/Errno.hh"
#include <string>
#include <stdio.h>
#include <unistd.h>

#include "t/main.hh"

using namespace fatalmind;

class MyTask: public Task {
	public:
		MyTask(CondVar<>& condvar, int sleep=1)
		: d_condvar(condvar)
		, d_sleep(sleep) 
		{};
		virtual ~MyTask() {};
	
		virtual void run() {
			sleep(d_sleep);
			d_condvar.lock();
			d_condvar.signal();
			d_condvar.unlock();
		}

	private:
		CondVar<> &d_condvar;
		int        d_sleep;
};

void TEST_CondVar_construct() {
	class CondVar<> c;
}

void TEST_CondVar_signal() {
	CondVar<> c;
	c.signal();
}

void TEST_CondVar_real() {
	CondVar<> c;
	Thread t;
	std::auto_ptr<Task> tsk(new MyTask(c));
	t.start(tsk);
	t.detach();
	c.lock();
	c.wait();
	c.unlock();
	//assertEqualT(void*, t.join(), NULL);
}

bool do_wait(CondVar<> &c, POSIX::timespec<> &to) {
	bool rv = false;
	c.lock();
	rv = c.wait(&to);
	c.unlock();
	return rv;
}
void TEST_CondVar_waitTimed() {
	CondVar<> c;
	POSIX::timespec<> to(1.05);	
	POSIX::timespec<> to2(6);	
	Thread t;
	std::auto_ptr<Task> tsk(new MyTask(c, 5));
	t.start(tsk);

	int i = 0;
	while (!do_wait(c, to)) {
		++i;
		assertLess(i, 6);
	}

	c.lock();
	assertFalse(c.wait(&to)); // must time out
	c.unlock();

	c.lock();
	assertFalse(c.wait(&to2)); // must not time out
	c.unlock();
}

