//********************************************************************
//*** Thread/Lock.t.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: Lock.t.cc,v 1.7 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "t/main.hh"
#include "Lock.hh"
#include <iostream>

using namespace fatalmind;

template <class Lock>
class memberLock {
	public:
		memberLock()
		: _lk()
		{};
		~memberLock() {};
		void doit() {_lk.lock(); _lk.unlock();}
	private:
		Lock _lk;
};

void TEST_PERF_memberLock_nopLock() {
	memberLock<NopLock<> > tst;
	for (int i = 0; i < 1000000; ++i) {
		tst.doit();
	}
}

void TEST_PERF_memberLock_fastLock() {
	memberLock<FastLock<> > tst;
	for (int i = 0; i < 1000000; ++i) {
		tst.doit();
	}
}

void TEST_PERF_memberLock_RecursiveLock() {
	memberLock<RecursiveLock<> > tst;
	for (int i = 0; i < 1000000; ++i) {
		tst.doit();
	}
}

template <class Lock>
class privateInhLock: private Lock {
		using Lock::lock;
		using Lock::unlock;
	public:
		privateInhLock()
		: Lock()
		{};
		~privateInhLock() {};
		void doit() {lock(); unlock();}
};

void TEST_PERF_privateInhLock_nopLock() {
	privateInhLock<NopLock<> > tst;
	for (int i = 0; i < 1000000; ++i) {
		tst.doit();
	}
}

void TEST_PERF_privateInhLock_fastLock() {
	privateInhLock<FastLock<> > tst;
	for (int i = 0; i < 1000000; ++i) {
		tst.doit();
	}
}

void TEST_PERF_privateInhLock_RecursiveLock() {
	privateInhLock<RecursiveLock<> > tst;
	for (int i = 0; i < 1000000; ++i) {
		tst.doit();
	}
}
