//********************************************************************
//*** Thread/Thread.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: Thread.cc,v 1.19 2007/09/20 11:17:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <pthread.h>
#include <sched.h>
#include <iostream>

#ifndef INCLUDED_THREAD_H
#include "Thread.hh"
#endif

#ifndef INCLUDED_ERRNO_H
#include "Exceptions/Errno.hh"
#endif

namespace fatalmind {

Thread::Thread()
: d_started(false)
{
}

Thread::~Thread() {
	try {
		join();
	} catch (...) {
	}
}

void* Thread::jumpinpoint(void* arg) {
	Task* t = reinterpret_cast<Task*>(arg);
	t->run();
	return NULL;
}

void Thread::start(Task* t) {
	Errno::ThrowOnError(
		pthread_create(&d_threadid, NULL, &Thread::jumpinpoint, t)
	);
	d_started = true;
}

void Thread::start(std::auto_ptr<Task> t) {
	d_task.reset(t);
	start(d_task.get());
}

void Thread::start(gc_ptr<Task, MultiThreadedModel<> > t) {
	d_task.reset(t);
	start(d_task.get());
}

void Thread::detach() {
	Errno::ThrowOnError(pthread_detach(d_threadid));
}

void Thread::join() {
	if (d_started) {
		void* ret;
		Errno::ThrowOnError(pthread_join(d_threadid, &ret));
		d_started = false;
	}
}

void Thread::yield() {
	Errno::ThrowOnError(sched_yield());
}

std::ostream& operator<<(std::ostream& os, const fatalmind::Thread& t) {
	return os << "Thread@[" << t.d_threadid << "]";
}

std::ostream& Thread::printSelf(std::ostream& os) {
	pthread_t id = pthread_self();	
	return os << "Thread@[" << id << "]";
}

} // namespace fatalmind

